﻿/*  Copyright © exc-jdbi 2021

    © Finite Field  2012

    © Finite Field  2012 is Free and Opensource Software
    
    www.vb-paradise.de/index.php/User/23082-exc-jdbi/
 
    github.com/exc-jdbi
*/

using System;
using System.Linq;
using System.Collections.Generic;

namespace FiniteFields
{
  public partial class FiniteField
  {
    /// <summary>
    /// Multipliziert zwei Polynome
    /// </summary>
    /// <param name="dividend">Polynom als Dividend</param>
    /// <param name="divisor">Polynom als Divisor</param>
    /// <param name="modulo">Begrenzt den Grad (Degree) des Polinoms auf den Modulowert</param>
    /// <returns></returns>
    private static (int[], int[]) FFPolynomDivision(
      int[] dividend, int[] divisor, int modulo)
    {
      var one = new int[] { 1 };
      var zero = new int[] { 0 };

      if (divisor.SequenceEqual(zero))
        throw new DivideByZeroException(nameof(divisor));

      if (dividend.SequenceEqual(zero))
        return (zero, zero.ToArray());

      if (divisor.SequenceEqual(one))
        return (dividend, zero);

      if (dividend.Length < divisor.Length)
        return (zero, dividend);

      int niveau = 0;
      int coefficient = 0;
      var tmp = new List<int>();
      var div = dividend.ToArray();
      var remainder = new List<int>(dividend.Length);
      var result = new int[dividend.Length - divisor.Length + 1];
      while (true)
      {
        niveau = div.Length - divisor.Length;
        remainder = new List<int>();
        coefficient = div[0] / divisor[0];
        coefficient = coefficient == 0 ? 1 : coefficient;
        for (int j = 0; j < div.Length; j++)
          if (j < divisor.Length)
            remainder.Add(div[j] - divisor[j] * coefficient);
          else remainder.Add(div[j]);

        remainder = ExtMod(remainder.ToArray(), modulo).ToList();
        var n = CleanseZeros(ref remainder);
        result[niveau] += coefficient;
        div = remainder.ToArray();
        if (remainder.Count < divisor.Length)
          break;

        if (remainder.Count == 1 && divisor.Length == 1)
          if (remainder[0] < divisor[0])
          {
            if (remainder[0] == 0) break;
            result[0] += CalcZeroRemainder(divisor[0],remainder[0],modulo);
            remainder[0] = 0;
            break;
          }
      }
      return (result.Reverse().ToArray(), remainder.ToArray());
    }

    private static int CalcZeroRemainder(int a, int r, int m)
    {
      int result = 1;
      while ((result++ * a) % m != r) ;
      return --result;
    }
  }
}
