﻿/*  Copyright © exc-jdbi 2021

    © Finite Field  2012

    © Finite Field  2012 is Free and Opensource Software
    
    www.vb-paradise.de/index.php/User/23082-exc-jdbi/
 
    github.com/exc-jdbi
*/

namespace FiniteFields
{
  public partial class FiniteField
  {
    /// <summary>
    /// Gibt zu Finite Field das Additiven Inverse zurück.
    /// </summary>
    /// <param name="finitefield">Das Finite Field</param>
    /// <returns>Gibt das AInv als Finite Field zurück</returns>
    private static FiniteField FFPolynomAdditiveInverse(FiniteField finitefield)
    {
      var addinverse = FFPolynomAdditiveInverse(finitefield.Value, finitefield);
      return new FiniteField(FromPolynom(addinverse, finitefield), finitefield);
    }

    /// <summary>
    /// Gibt zu einem Value im Finite Field das Additiven Inverse zurück.
    /// </summary>
    /// <param name="value">Polynom</param>
    /// <param name="reference">Referenz Finite Field</param>
    /// <returns>Gibt das AInv als Polynom zurück.</returns>
    private static int[] FFPolynomAdditiveInverse(int value, FiniteField reference)
    {
      //Sehr einfach und sicher gelöst.
      if (value == 0) return new[] { 0 };
      if (value < reference.Generator)
        ToPolynom(ExtMod(reference.Generator - value, reference.Generator), reference);

      int tmp = value;
      while (tmp-- % reference.Generator != 0) ;
      tmp++;

      var polynomial = ToPolynom(reference.Polynomial, reference);
      var apolinomvalue = ToPolynom(tmp, reference);
      var bpolinomvalue = ToPolynom(reference.Generator - 1, reference);
      var offset = FromPolynom(FFPolynomMultiplication(apolinomvalue, bpolinomvalue, polynomial, reference.Generator).Item2, reference);

      tmp = ExtMod(reference.Generator - ExtMod(value, reference.Generator), reference.Generator);
      return ToPolynom(ExtMod(tmp + offset, reference.Order), reference);
    }

    /// <summary>
    /// Gibt zu einem Value im Finite Field das Multiplikative Inverse zurück.
    /// </summary>
    /// <param name="value">Value als Dezimalwert</param>
    /// <param name="reference">Referenz Finite Field</param>
    /// <returns>Gibt das MInv als Finite Field zurück.</returns>
    private static FiniteField FFPolynomMultiplicativeInverse(int value, FiniteField reference)
    => FFPolynomMultiplicativeInverse(new FiniteField(value, reference));

    /// <summary>
    /// Gibt zu einem Finite Field das Multiplikative Inverse zurück.
    /// </summary>
    /// <param name="finitefield">Referenz Finite Field</param>
    /// <returns>Gibt das MInv als Finite Field zurück.</returns>
    private static FiniteField FFPolynomMultiplicativeInverse(FiniteField finitefield)
    {
      if (finitefield == 0) return finitefield;
      if (finitefield == 1) return finitefield;

      //Das Irreduzible Polynom ist immer das grössere
      //Damit das Resultat von value in Item2 ist, genau so rein lassen.
      if (finitefield.Exponent == 1)
        return FFPolynomExtEEaNumeral(finitefield);

      var value = ToPolynom(finitefield.Value, finitefield);
      var idp = ToPolynom(finitefield.Polynomial, finitefield);
      var exteea = FFPolynomExtEEa(value, idp, finitefield);
      var inverse = FromPolynom(exteea.Item2, finitefield);
      return new FiniteField(inverse, finitefield);
    }




    /* ************ ************ ************ ************ ************ ************ 
       ************ ************ ************ ************ ************ ************ 
        Eine weitere sichere Möglichkeit, um die Multiplicative Inverse zu finden.
        Sehr hohe iterative Kosten, da das Polynom "1" gesucht wird.
       ************ ************ ************ ************ ************ ************ 
       ************ ************ ************ ************ ************ ************ */

    //private static (int, int[]) MultReciprocal(int xpolynomvalue, FiniteField reference)
    //{
    //  var xpolynom = ToPolynom(ExtMod(xpolynomvalue, reference.Order), reference);
    //  return MultReciprocal(xpolynom, reference);
    //}
    //private static (int, int[]) MultReciprocal(int[] xpolynom, FiniteField reference)
    //{
    //  int[] tmp = null;
    //  (int[], int[]) mult = default;
    //  int increment = 0, result = -1;
    //  var polynomial = ToPolynom(reference.Polynomial, reference);
    //  var finish = reference.Order + 1;
    //  while (result != 1 && increment < finish)
    //  {
    //    tmp = ToPolynom(ExtMod(increment++, reference.Order), reference);
    //    mult = FFPolynomMultiplication(xpolynom, tmp, polynomial, reference.Generator);
    //    if (reference.Exponent == 1)
    //      result = FromPolynom(mult.Item1, reference);
    //    else result = FromPolynom(mult.Item2, reference);
    //  }
    //  if (increment == finish)
    //    throw new ArgumentException($"{nameof(polynomial)} is not useful.");
    //  return (increment - 1, tmp);
    //}
  }
}
