﻿/*  Copyright © exc-jdbi 2021

    © Finite Field  2012

    © Finite Field  2012 is Free and Opensource Software
    
    www.vb-paradise.de/index.php/User/23082-exc-jdbi/
 
    github.com/exc-jdbi
*/


using System;
using System.Linq;
using NumeralSystemConverter;
using System.Collections.Generic;

namespace FiniteFields
{
  public partial class FiniteField
  {

    public int[][] ToFFMultiplicationTable()
    {
      if (this.Order > MAX_TABLE) return default;
      var ffpolynom = this.ToFFPolynoms();
      var irrpolynomial = ToPolynom(this.Polynomial,this.Generator);
      var ffpolynomtable = this.ToFFMultiplicationPolynomTable(ffpolynom, irrpolynomial);
      return this.ToFFTable(ffpolynomtable, irrpolynomial);
    }

    public int[][][] ToFFMultiplicationPolynomTable()
    {
      if (this.Order > MAX_TABLE) return default;
      var ffpolynom = this.ToFFPolynoms();
      var irrpolynomial = ToPolynom(this.Polynomial,this.Generator);
      return this.ToFFMultiplicationPolynomTable(ffpolynom, irrpolynomial);
    }

    public int[][] ToFFAdditionTable()
    {
      if (this.Order > MAX_TABLE) return default;
      var ffpolynom = this.ToFFPolynoms();
      var irrpolynomial = ToPolynom(this.Polynomial,this.Generator);
      var ffpolynomtable = this.ToFFAdditionPolynomTable(ffpolynom, irrpolynomial);
      return this.ToFFTable(ffpolynomtable, irrpolynomial);
    }

    public int[][][] ToFFAdditionPolynomTable()
    {
      if (this.Order > MAX_TABLE) return default;
      var ffpolynom = this.ToFFPolynoms();
      var irrpolynomial = ToPolynom(this.Polynomial,this.Generator);
      return this.ToFFAdditionPolynomTable(ffpolynom, irrpolynomial);
    }

    private int[][] ToFFTable(int[][][] ffpolynomtable, int[] irrpolynomial)
    {
      //if (this.Order > MAX_TABLE) return default;
      int sum = 0;
      var dimension = ffpolynomtable.Length;
      var result = new int[dimension][];
      for (int i = 0; i < dimension; i++)
      {
        var tmp = new int[dimension];
        for (int j = 0; j < dimension; j++)
        {
          Array.Reverse(ffpolynomtable[i][j]);
          sum = ffpolynomtable[i][j][0];
          for (int z = 1; z < ffpolynomtable[i][j].Length; z++)
          {
            if (ffpolynomtable[i][j][z] == 0 && z == 0) continue;
            sum += ffpolynomtable[i][j][z] * (int)Math.Pow(this.Generator, z);
          }
          Array.Reverse(ffpolynomtable[i][j]);
          tmp[j] = sum;
        }
        result[i] = tmp;
      }
      return result;
    }

    private int[][][] ToFFMultiplicationPolynomTable(int[][] ffpolynom, int[] irrpolynomial)
    {
      //Multiplikationstablle
      //if (this.Order > MAX_TABLE) return default;
      var dimension = ffpolynom.Length;
      var result = new int[dimension][][];
      result[0] = Enumerable.Repeat(0, dimension).Select(x => new[] { x }).ToArray();
      for (int i = 1; i < dimension; i++)
      {
        var tmp = new int[dimension][];
        tmp[0] = new[] { 0 };
        for (int j = 1; j < dimension; j++)
        {
          if (this.Exponent == 1)
            tmp[j] = FFPolynomMultiplication(ffpolynom[i], ffpolynom[j], irrpolynomial, this.Generator).Item1;
          else tmp[j] = FFPolynomMultiplication(ffpolynom[i], ffpolynom[j], irrpolynomial, this.Generator).Item2;
        }
        result[i] = tmp;
      }
      return result;
    }

    private int[][][] ToFFAdditionPolynomTable(int[][] ffpolynom, int[] irrpolynomial)
    {
      //Additionstabelle
      //if (this.Order > MAX_TABLE) return default;
      var dimension = ffpolynom.Length;
      var result = new int[dimension][][];
      for (int i = 0; i < dimension; i++)
      {
        var tmp = new int[dimension][];
        for (int j = 0; j < dimension; j++)
        {
          if (this.Exponent == 1)
            tmp[j] = FFPolynomAddition(ffpolynom[i], ffpolynom[j], irrpolynomial, this.Generator).Item1;
          else tmp[j] = FFPolynomAddition(ffpolynom[i], ffpolynom[j], irrpolynomial, this.Generator).Item2;
        }
        result[i] = tmp;
      }
      return result;
    }

    private int[][] ToFFPolynoms()
    {
      //Erstellt alle Polynome für die Table-Erstellung

      //if (this.Order > MAX_TABLE) return default;
      var result = new List<int[]>(this.Order) { new[] { 0 } };
      for (int i = 1; i < this.Order; i++)
        result.Add(NSC.Converts(
          NSC.ToNumbers(i), 10, this.Generator));
      return result.ToArray();
    }
  }
}
