﻿/*  Copyright © exc-jdbi 2021

    © Finite Field  2012

    © Finite Field  2012 is Free and Opensource Software
    
    www.vb-paradise.de/index.php/User/23082-exc-jdbi/
 
    github.com/exc-jdbi
*/

using System;

namespace FiniteFields
{
  /// <summary>
  /// Repräsentiert ein Endlicher Körper
  /// </summary>
  public partial class FiniteField
  {

    /// <summary>
    /// C-Tor
    /// </summary>
    /// <param name="_finiteField">FF-Element</param>
    public FiniteField(FiniteField _finiteField)
      : this(_finiteField.Generator, _finiteField.Exponent,
             _finiteField.Polynomial, _finiteField.Value)
    {
    }

    /// <summary>
    /// C-Tor
    /// </summary>
    /// <param name="_value">Value als Dezimalwert</param>
    /// <param name="_ffreference">FF-Element als Referenz</param>
    public FiniteField(int _value, FiniteField _ffreference)
      : this(_ffreference.Generator, _ffreference.Exponent,
             _ffreference.Polynomial, _value)
    {
    }

    /// <summary>
    /// C-Tor
    /// </summary>
    /// <param name="_generator">characteristic</param>
    /// <param name="_value">Decimal value</param>
    public FiniteField(int _generator, int _value)
      : this(_generator, 1, 1, _value)
    {
    }

    /// <summary>
    /// C-Tor
    /// </summary>
    /// <param name="_generator">characteristic</param>
    /// <param name="_exponent">degree</param>
    /// <param name="_polynomial">Irreducible polynomial</param>
    public FiniteField(int _generator, int _exponent, int _polynomial)
      : this(_generator, _exponent, _polynomial, 0)
    {
    }
    /// <summary>
    /// C-Tor
    /// </summary>
    /// <param name="_generator">characteristic</param>
    /// <param name="_exponent">degree</param>
    /// <param name="_polynomial">Irreducible polynomial</param>
    /// <param name="_value">Decimal value</param>
    public FiniteField(int _generator, int _exponent, int _polynomial, int _value)
    {
      if (_generator < 2)
        throw new InvalidOperationException($"{nameof(_generator)} < 2");

      if (_exponent < 1 || _polynomial < 1)
        throw new InvalidOperationException($"{nameof(_exponent)} || {nameof(_exponent)} < 1");

      this.Exponent = _exponent;
      if (!this.IsPrime(_generator))
        throw new InvalidOperationException(
          "For FF-Calculations - Generator must be a Prime");

      this.Generator = _generator;
      this.Polynomial = _exponent == 1 ? 1 : _polynomial;
      this.Order = (int)Math.Pow(_generator, _exponent);
      this.Value = _value;
    }

    private bool IsPrime(int b)
    {
      for (int i = 2; i < (int)Math.Pow(b, 1d / 2d) + 1; i++)
        if (b % i == 0) return false;
      return true;
    }
  }
}
