﻿/*  Copyright © exc-jdbi 2021

    © Finite Field  2012

    © Finite Field  2012 is Free and Opensource Software
    
    www.vb-paradise.de/index.php/User/23082-exc-jdbi/
 
    github.com/exc-jdbi
*/


using NumeralSystemConverter;

namespace FiniteFields
{
  public partial class FiniteField
  {
    /// <summary>
    /// Erstellt das Polynom für eine bestimme 
    /// Basis (Generator, Degree)
    /// </summary>
    /// <param name="number">Nummer at decimal system</param>
    /// <param name="generator">Target Base (Generator, Degree)</param>
    /// <returns>Polynom als Array</returns>
    public static int[] ToPolynom(int number, int generator)
    {
      return NSC.Converts(
         NSC.ToNumbers(number), 10, generator);
    }

    /// <summary>
    /// Erstellt das Polynom für das gleiche Finite Field System, 
    /// (Reference FF) welches miteingegeben wird.
    /// </summary>
    /// <param name="number">Dezimalwert</param>
    /// <param name="equalsfinitefield">Reference FF</param>
    /// <returns>Polynom als Array</returns>
    public static int[] ToPolynom(int number, FiniteField equalsfinitefield)
    {
      if (number == 0) return new[] { 0 };
      return NSC.Converts(
         NSC.ToNumbers(number), 10, equalsfinitefield.Generator);
    }

    /// <summary>
    /// Gibt den Dezimalwert zu einem Polynom für ein 
    /// bestimmtes Finite Field System zurück
    /// </summary>
    /// <param name="polynom">Polynom als Array</param>
    /// <param name="generator">CurrentBase (Generator, Degree)</param>
    /// <returns>Polynom als Dezimalwert</returns>
    public static int FromPolynom(int[] polynom, int generator)
    => NSC.NumbersToI32(NSC.Converts(polynom, generator, 10));


    /// <summary>
    /// Gibt den Dezimalwert zu einem Polynom für ein 
    /// bestimmtes Finite Field System zurück
    /// </summary>
    /// <param name="polynom">Polynom als Array</param>
    /// <param name="equalsfinitefield">Reference FF</param>
    /// <returns>Polynom als Dezimalwert</returns>
    public static int FromPolynom(int[] polynom, FiniteField equalsfinitefield)
    => NSC.NumbersToI32(NSC.Converts(polynom, equalsfinitefield.Generator, 10));

    /// <summary>
    /// Erstellt eine Kopie des Finite Fields
    /// </summary>
    /// <param name="_finitefield">FF-System, dass kopiert werden soll.</param>
    /// <returns></returns>
    public static FiniteField Clone(FiniteField _finitefield)
    => new FiniteField(_finitefield);

    /// <summary>
    /// Prüft, ob zwei FF-Systeme die gleichen Dimensionen
    /// Generator (Characteristic) und Exponent (Degree)
    /// besitzen.
    /// </summary>
    /// <param name="finitefield1">FF-System</param>
    /// <param name="finitefield2">FF-System</param>
    /// <returns>true wen gleich, ansonsten false.</returns>
    public static bool EqualsDimensions(
      FiniteField finitefield1, FiniteField finitefield2)
    {
      if (object.ReferenceEquals(finitefield1, finitefield2)) return true;
      return finitefield1.Generator == finitefield2.Generator &&
             finitefield1.Exponent == finitefield2.Exponent;
    }

    /// <summary>
    /// Prüft, ob zwei FF-Systeme den gleichen 
    /// Value als Dezimalwert besitzen. 
    /// <para></para>
    /// WICHTIG: Keine Moduloprüfung!
    /// </summary>
    /// <param name="finitefield1"></param>
    /// <param name="finitefield2"></param>
    /// <returns>true wen gleich, ansonsten false.</returns>
    public static bool EqualsValues(
      FiniteField finitefield1, FiniteField finitefield2)
    => finitefield1.Value == finitefield2.Value;



    /// <summary>
    /// Konvertiert ein Polynom in ein anderes Basissystem.
    /// </summary>
    /// <param name="polynom">Polnom als Array</param>
    /// <param name="currentgenerator">Jetziges Basissystem</param>
    /// <param name="targetgenerator">Gewünschtes Basissystem</param>
    /// <returns></returns>
    public static int[] ConvertPolynom(
      int[] polynom, int currentgenerator, int targetgenerator)
    {
      return NSC.Converts(
          NSC.Converts(polynom, currentgenerator, 10),
          10, targetgenerator);
    }
    /// <summary>
    /// Erweiterte Modulo-Berechnung
    /// </summary>
    /// <param name="number">number</param>
    /// <param name="modulo">modulo</param>
    /// <returns></returns>
    public static int ExtMod(int number, int modulo)
    {
      number %= modulo;
      return (int)(((long)number + modulo) % modulo);
    }

    /// <summary>
    /// Richtet ein Polynom mit negativen Koeefizienten, in ein
    /// Polynom mit positiven Koeffizienten, mit dem gleichen Wert um.
    /// </summary>
    /// <param name="polynom">Das gewünschte Polnom das gerichtet werden soll</param>
    /// <param name="generator">CurrentBase (Generator, Degree)</param>
    /// <returns>Polynom mit positiven Koeeffizienten</returns>
    public static int[] ExtMod(int[] polynom, int modulo)
    //=> ToPolynom(FromPolynom(polynom, generator), generator);
    {
      for (int i = 0; i < polynom.Length; i++)
        polynom[i] = ExtMod(polynom[i], modulo);
      return polynom;
    }

    /// <summary>
    /// Richtet zwei Polynome mit negativen Koeefizienten, in
    /// Polynome mit positiven Koeffizienten, mit den gleichen Werten um.
    /// </summary>
    /// <param name="polynom"></param>
    /// <param name="ffreference"></param>
    /// <returns></returns>
    public static int[] ExtMod(int[] polynom, FiniteField ffreference)
    => ExtMod(polynom, ffreference.Generator);

    /// <summary>
    /// Richtet zwei Polynome mit negativen Koeefizienten, in
    /// Polynome mit positiven Koeffizienten, mit den gleichen Werten um.
    /// </summary>
    /// <param name="polynom1">Das gewünschte Polnom das gerichtet werden soll</param>
    /// <param name="polynom2">Das gewünschte Polnom das gerichtet werden soll</param>
    /// <param name="generator">CurrentBase (Generator, Degree)</param>
    /// <returns>Polynoms mit positiven Koeeffizienten</returns>
    public static (int[], int[]) ExtMod(int[] polynom1, int[] polynom2, int modulo)
    => (ExtMod(polynom1, modulo), ExtMod(polynom2, modulo));

    /// <summary>
    /// Richtet zwei Polynome mit negativen Koeefizienten, in
    /// Polynome mit positiven Koeffizienten, mit den gleichen Werten um.
    /// </summary>
    /// <param name="polynom1">Das gewünschte Polnom das gerichtet werden soll</param>
    /// <param name="polynom2">Das gewünschte Polnom das gerichtet werden soll</param>
    /// <param name="ffreference">Reference FF</param>
    /// <returns></returns>
    public static (int[], int[]) ExtMod(int[] polynom1, int[] polynom2, FiniteField ffreference)
    => ExtMod(polynom1, polynom2, ffreference.Generator);


    /// <summary>
    /// Gibt die numerische Multiplikative Inverse zurück.
    /// </summary>
    /// <param name="a">Sinnvollerweise hier den grösseren Wert.</param>
    /// <param name="m">Der zweite decimale Wert.</param>
    /// <returns></returns>
    public static int ModInverse(int a, int m)
    {
      var guv = ExtGcdIterativ(a, m);
      return guv.Item2 % m;
    }


  }
}
